#! /usr/bin/python

"""//Web service to place an order for a batch//

- > An order must be placed when a user decide to allocate some credit to one of his batch to enhance its QoS. 
 
- File: order.py 
 
- Arguments - Using the HTTP GET method:
 - batch_id -- The batch identifier, as a string
 - user_id -- The user identifier, as an integer
 - credit -- The number of credits to allocate to the order, as a positive integer
 - > If some arguments are missing, a web form will be displayed.
 
- Displays a Web page containing:
 - ``<div class="qos">True</div>`` if the order is accepted     
 - ``<div class="qos">False</div>`` if the order is denied
 
- Notes: 
 - An order will be denied if there are not enough credits on the user's institution account.
 - A database configuration file named "db.cfg" must be present in the directory.

"""
import sys
import os
sys.path.append(os.path.join(sys.path[0],".."))
os.chdir(sys.path[0])
from tools import db_func
from tools import cgi_func
from tools import log_func
log=log_func.get_spequlos_log(__file__)


def order(conn,batch_id,user_id,credit):
    """Function implementing the  order operation

    Arguments:
    conn -- The CreditSystem's DB connection, as a MySQL.connections.Connection object
    batch_id -- The batch identifier, as a string
    user_id --- The user identifier, as an integer
    credit -- The number of credits to allocate, as a positive integer
    
    The function returns True if the order is accepted, or False otherwise

    """
    if not isinstance(conn,db_func.db_connection) or conn.open==0:
        log.error("The conn argument must be an open connection to the database !")
        return None      
    if not isinstance(credit,int) or credit<=0:
        log.error("The credit argument must be a positive integer !")
        return None
    if not isinstance(user_id,int):
        log.error("The user_id argument must be an integer !")
        return None
    if not isinstance(batch_id,str):
        log.error("The batch_id argument must be a string !")
        return None        

    c=conn.cursor()

    log.info("Retrieving user institution...")
    try:
        c.execute("SELECT institution_id FROM QoSUser WHERE user_id=%d" % user_id)
    except db_func.DBError as e:
        log.error("Error during the SQL command execution !")
        log.error(e)
        return None      
    res = c.fetchall()
    
    if len(res)==0:
        log.error("No such user identifier in the database : %d)" % user_id )
        return None
    elif len(res)>1:
        log.error("Several users with identifier %d in the database ! 'user_id' should be QoSUser table PRIMARY KEY"% user_id)
        return None
    
    try:
        institution_id = int(res[0][0])
    except ValueError as e:
        log.error("'institution_id' is not an integer in the table QoSUser")  
        log.error(e)  
        return None          
    
    log.info("User institution is %d" % institution_id)

    log.info("Retrieving information on institution %d" % institution_id )
    try:
        c.execute("SELECT credit_balance,cloud_id FROM Institution WHERE institution_id=%d" % institution_id)
    except db_func.DBError as e:
        log.error("Error during the SQL command execution !")
        log.error(e)
        return None  
    res = c.fetchall()
    
    if len(res)==0:
        log.error("No such institution identifier in the database : %d" % institution_id)
        return None
    if len(res)>1:
        log.error("Several institution with identifier %d in the database ! 'institution_id' should be Institution table PRIMARY KEY" % institution_id)
        return None
 
    try:
        institution_credit_balance = int(res[0][0])
        if institution_credit_balance<0:
            raise ValueError("Negative credit_balance !")
    except ValueError as e:
        log.error("'credit_balance' is not an integer in the table Institution")  
        log.error(e)
        return None        
    
    try:        
        cloud_id = int(res[0][1])
    except ValueError as e:
        log.error("'cloud_id' is not an integer in the table Institution")  
        log.error(e)
        return None        
    
    log.info("Institution has %d credits and is associated to cloud %d" % (institution_credit_balance,cloud_id))

    if institution_credit_balance<credit:
        log.error("Institution has not enough credit (only %d)" % institution_credit_balance)
        return False

    log.info("Updating institution credit balance")
    try:
        c.execute("UPDATE Institution SET credit_balance=%d WHERE institution_id=%d" \
            % (institution_credit_balance-credit,institution_id))
    except db_func.DBError as e:
        log.error("Error during the SQL command execution !")
        log.error(e)
        return None  
    log.info("Done!")

    log.info("Checking if there is already an order for this batch_id...")
    try:
        c.execute("SELECT batch_id,credit_balance,credit_funded FROM QoSOrder WHERE batch_id='%s'" % batch_id)
    except db_func.DBError as e:
        log.error("Error during the SQL command execution !")
        log.error(e)
        return None  
    res=c.fetchall()

    if len(res)>1:
        log.error("Several orders associated with batch identifier %d in the database ! 'batch_id' should be QoSOrder PRIMARY KEY !" % batch_id)
        return None            
    elif len(res)==1:
        order_batchid = res[0][0]
        try:
            order_credit_balance = int(res[0][1])
            if order_credit_balance<0:
                raise ValueError("Negative credit_balance")
        except ValueError as e:
            log.error("The 'credit_balance' in QoSOrder table must be a positive or zero integer !")
            log.error(e)
            return None            
        try:
            order_credit_funded = int(res[0][2])
            if order_credit_balance<0:
                raise ValueError("Negative credit_funded")
        except ValueError as e:
            log.error("The 'credit_funded' in QoSOrder table must be a positive or zero integer !")
            log.error(e)
            return None            
        
        log.warning("An order already exists for this batch id : %s, adding credits to it anyway" % order_batchid)
        try:
            c.execute("UPDATE QoSOrder SET credit_balance=%d,credit_funded=%d WHERE batch_id='%s'" % \
                (order_credit_balance+credit,order_credit_funded+credit,batch_id))
        except db_func.DBError as e:
            log.error("Error during the SQL command execution !")
            log.error(e)
            return None                  
    else:
        log.info("No existing order, creating one")
        try:
            c.execute("INSERT INTO QoSOrder VALUES ('%s',%d,%d,%d,%d)" % (batch_id,credit,credit,cloud_id,user_id))
        except db_func.DBError as e:
            log.error("Error during the SQL command execution !")
            log.error(e)
            return None              
        log.info("Done")
        
    log.info("Order completed")
    return True

if __name__=="__main__":
    cgi_func.print_header()

    if (cgi_func.get_form_value('batch_id') is None) or  (cgi_func.get_form_value('user_id') is None) \
	    or (cgi_func.get_form_value('credit') is None):
        cgi_func.print_form(['batch_id','user_id','credit'])

    else:
        try:
            batch_id = cgi_func.get_form_value('batch_id')
            user_id = int(cgi_func.get_form_value('user_id'))
            credit = int(cgi_func.get_form_value('credit'))
        except ValueError as e:
            log.error("user_id and credit arguments must be integers")
            cgi_func.print_form(['dg_id','institution_id','credit']) 
        else:
            conn=db_func.connect(db_func.read_db_config("db.cfg"))
            cgi_func.print_result( order(conn,batch_id,user_id,credit))
            db_func.disconnect(conn)
        
    cgi_func.print_footer()

