#! /usr/bin/python

"""//Functions used by the scheduler to remotely configure Desktop Grids to enable QoS//

- File: dg_rpc.py

"""

import sys
import os
sys.path.append(os.path.join(sys.path[0],".."))
os.chdir(sys.path[0])
from tools import web_func
from tools import conf_func
from tools import log_func
log=log_func.get_spequlos_log(__file__)


def preconfigure_QoS(batch_id,dg_id,dg_config_file):
    """//Preconfigure a Desktop Grid to be able to handle QoS//

    - > The preconfiguration is a configuration step that must be performed in the DG before the batch submission, i.e. when the user calls the start_qos function

    - Arguments:
     - batch_id -- The batch identifier, as a string
     - dg_id -- The supported Desktop Grid identifer, as a string
     - dg_config_file -- The Desktop Grid configuration file, as a string
    
    - The function returns some datas, as a string, specific to the DG technology, as output of the preconfiguration (for XWHEP its the group URI associated with the batch_id) or True if there is no specific data or None is the preconfiguration failed
    
    """
    if not isinstance(batch_id,str):
        log.error("The batch_id argument must be a string")
        raise Exception("The batch_id argument must be a string")
    if not isinstance(dg_id,str):
        log.error("The dg_id argument must be a string")
        raise Exception("The dg_id argument must be a string")        
    if not isinstance(dg_config_file,str):
        log.error("The dg_config_file argument must be a string")
        raise Exception("The dg_config_file argument must be a string")
 
    log.info("Reading DG configuration file")  
    dg_params = conf_func.read_config(dg_config_file)
    
    if not "DG_TYPE" in dg_params:
        log.error("No DG_TYPE parameter in DG configuration file %s" % dg_config_file)
        raise Exception("No DG_TYPE parameter in %s" % dg_config_file)
    if not "DG_PLUGIN_URL" in dg_params:
        log.error("No DG_PLUGIN_URL parameter in DG configuration file %s" % dg_config_file)
        raise Exception("No DG_PLUGIN_URL parameter in %s" % dg_config_file)
    dg_type=dg_params["DG_TYPE"]
    dg_plugin_url=dg_params["DG_PLUGIN_URL"]   

    log.info("Checking the DG technology used")
    if dg_type=="XWHEP":    
        log.info("For XWHEP, we have to create a job group associated to our batch. Calling corresponding plugin...")

        if dg_plugin_url[-1:] == "/":
            log.debug("Removing trailing slash from dg_plugin_url")
            dg_plugin_url=dg_plugin_url[0:-1]

        url=dg_plugin_url+"/XWHEP-addgroup.php?batch_id="+batch_id
        res=web_func.call(url)
        
        if len(res)!=1:
            log.error("There should be 1 results from url %s" % url)
            return None
        if res[0]=="False":
            log.error("The %s server at %s returned an error with batch_id %s" % (dg_type,dg_plugin_url,batch_id))
            return None
        else:
            uri = res[0]
            log.info("The %s server at %s returned the URI %s for batch_id %s" % (dg_type,dg_plugin_url,uri,batch_id))
            return uri
    else:
        log.info("No preconfiguration needed for %s grid technology" % dg_type)
        return True




def configure_qos(batch_id,dg_id,dg_config_file,cw_instance_id):
    """//Configure a Desktop Grid to start QoS//

    - > The configuration is a step performed once the requierment to start QoS are filled (ie. credit ordered, oracle agreement, cloudworker started, etc.)

    - Arguments:
     - batch_id -- The batch identifier, as a string
     - dg_id -- The supported Desktop Grid identifer, as a string
     - dg_config_file -- The Desktop Grid configuration file, as a string
     - cw_instance_id -- The cloud worker identifier, as an integer. Currently unused.

    - The function returns True if the configuration succeed or None otherwise
    
    """     
    if not isinstance(batch_id,str):
        log.error("The batch_id argument must be a string")
        raise Exception("The batch_id argument must be a string")
    if not isinstance(dg_id,str):
        log.error("The dg_id argument must be a string")
        raise Exception("The dg_id argument must be a string")        
    if not isinstance(dg_config_file,str):
        log.error("The dg_config_file argument must be a string")
        raise Exception("The dg_config_file argument must be a string")

    log.info("Reading DG configuration file")  
    dg_params = conf_func.read_config(dg_config_file)
    
    if not "DG_TYPE" in dg_params:
        log.error("No DG_TYPE parameter in DG configuration file %s" % dg_config_file)
        raise Exception("No DG_TYPE parameter in %s" % dg_config_file)
    if not "DG_PLUGIN_URL" in dg_params:
        log.error("No DG_PLUGIN_URL parameter in DG configuration file %s" % dg_config_file)
        raise Exception("No DG_PLUGIN_URL parameter in %s" % dg_config_file)
    dg_type=dg_params["DG_TYPE"]
    dg_plugin_url=dg_params["DG_PLUGIN_URL"]   
 
    log.info("Checking the DG technology used")
    if dg_type=="BOINC":    
        log.info("For BOINC, we have to ask for association between uncompleted workunit and cloud worker. Calling corresponding plugin...")

        if dg_plugin_url[-1:] == "/":
            log.debug("Removing trailing slash from dg_plugin_url")
            dg_plugin_url=dg_plugin_url[0:-1]

        url=dg_plugin_url+"/assign_batch_to_cloud?batch="+batch_id
        res=web_func.call(url)
        
        if len(res)!=1:
            log.error("There should be 1 results from url %s" % url)
            return None
        if res[0]!="True":
            log.error("The %s server at %s returned an error with batch_id %s" % (dg_type,dg_plugin_url,batch_id))
            return None
        else:
            log.info("Configuration succeed !")
            return True
    else:
        log.info("No preconfiguration needed for %s grid technology" % dg_type)
        return True


def unconfigure_QoS(batch_id,dg_id,dg_config_file,batch_data,cw_instance_id=-1,cw_data=''):
    """//Unconfigure a Desktop Grid when QoS operation finished//

    - > The unconfiguration is a configuration step that must be performed in the DG when the QoS operations related to a batch is finnished, i.e. when the batch is completed or when no more resources are available to support QoS

    - Arguments:
     - batch_id -- The batch identifier, as a string
     - dg_id -- The supported Desktop Grid identifer, as a string
     - dg_config_file -- The Desktop Grid configuration file, as a string
     - batch_data -- Some data associated to the batch, for DG technology-specific information, as a string (used to store the group URI associated to the batch in XWHEP), as a string
     - cw_instance_id -- The CloudWorker identifier, as an integer. Currently unused in this function.
     - cw_data -- Some data associated to the cloud worker, for technology-specific information, as a string (used to store the instance name for EC2), as a string. Currently unused in this function.

    - The function returns True if the unconfiguration succeed or None otherwise
    
    """     
    if not isinstance(batch_id,str):
        log.error("The batch_id argument must be a string")
        raise Exception("The batch_id argument must be a string")
    if not isinstance(dg_id,str):
        log.error("The dg_id argument must be a string")
        raise Exception("The dg_id argument must be a string")        
    if not isinstance(dg_config_file,str):
        log.error("The dg_config_file argument must be a string")
        raise Exception("The dg_config_file argument must be a string")   
    if not isinstance(batch_data,str):
        log.error("The batch_data argument must be a string")
        raise Exception("The batch_data argument must be a string")            
    if not isinstance(cw_data,str):
        log.error("The cw_data argument must be a string")
        raise Exception("The cw_data argument must be a string")            
 
    log.info("Reading DG configuration file")  
    dg_params = conf_func.read_config(dg_config_file)
    
    if not "DG_TYPE" in dg_params:
        log.error("No DG_TYPE parameter in DG configuration file %s" % dg_config_file)
        raise Exception("No DG_TYPE parameter in %s" % dg_config_file)
    if not "DG_PLUGIN_URL" in dg_params:
        log.error("No DG_PLUGIN_URL parameter in DG configuration file %s" % dg_config_file)
        raise Exception("No DG_PLUGIN_URL parameter in %s" % dg_config_file)
    dg_type=dg_params["DG_TYPE"]
    dg_plugin_url=dg_params["DG_PLUGIN_URL"]   

    log.info("Checking which DG technology is used")
    if dg_type=="XWHEP":  
        
        if dg_plugin_url[-1:] == "/":
            log.debug("Removing trailing slash from dg_plugin_url")
            dg_plugin_url=dg_plugin_url[0:-1]

        url=dg_plugin_url+"/XWHEP-rmgroup.php?batch_id_uri="+batch_data
        log.info("For XWHEP, we have to remove the job group associated to our batch. Calling corresponding plugin at url %s" % url)

        res=web_func.call(url)
        if len(res)!=1:
            log.error("There should be 1 results from url %s" % url)
            return None
        if res[0]!="True":
            log.error("The %s server at %s returned an error with uri %s" % (dg_type,dg_plugin_url,batch_data))
            return None
        else:
            log.info("The %s server at %s deleted the URI %s associated to batch_id %s" % (dg_type,dg_plugin_url,batch_data,batch_id))
            return True 
    else:
        log.info("No preconfiguration needed for %s grid technology" % dg_type)
        return True



if __name__=="__main__":
    import sys
    batch_id,dg_id,dg_config_file = "test_batch_id","test_dg",sys.argv[1]

    print "Press enter to preconfigure DG"
    x=raw_input()
    batch_data = preconfigure_QoS(batch_id,dg_id,dg_config_file)

    print "Press enter to configure DG"
    x=raw_input()
    configure_qos(batch_id,dg_id,dg_config_file,0)
    
    print "Press enter to unconfigure DG"
    x=raw_input()
    unconfigure_QoS(batch_id,dg_id,dg_config_file,batch_data)
    
    print "Done"
