#! /usr/bin/python

"""//Program that monitor Cloud Worker currently running//

- > This program manages Cloud Worker currently running, by billing the resources being spent, and stopping them if needed. It should be started periodically to perform monitoring operations. 

- File: monitor_cloud_workers.py

- Notes: 
 - A database configuration file named "db.cfg" must be present in the directory. 
 - A file named "creditsystem.cfg" containing a line "address=<url>", with <url> the url to access to the Credit System web server, must be present in the directory. 

- Algorithm:
``` 
For each of running cloud workers, this function checks 
  if the worker finished his job, 
      stop it
  else if there is still some credits in the batch order, 
      bill the resource spent
  else 
      stop the worker
  if the worker stopped and no other workers are working 
                                    on the worker's batch, 
      ask the Credit System to pay the order  
```

"""

import sys
import os
sys.path.append(os.path.join(sys.path[0],".."))
os.chdir(sys.path[0])
from tools import conf_func
from tools import web_func
from tools import db_func
from tools import log_func
log=log_func.get_spequlos_log(__file__)

import cloudworker
import oracle_rpc
import creditsystem_rpc
import dg_rpc


def monitor_cloud_workers(db):
    """//Function implementing the cloud worker monitoring//

    - Arguments:
     - db -- The scheduler's DB connection, as a SpeQuloS_DB object

    """    
    log.info("Retrieving running Cloud worker list...")
    cworkers = db.sql_select(("CloudService","CloudWorker","QoSBatch","DesktopGrid"), \
    ('cloud_worker_id','cloud_service_id','cloud_service_conf_file','handler','cloud_worker_data', \
    'batch_id','batch_data','dg_id','dg_conf_file'),
    "batch_id IS NOT NULL")
    
    if len(cworkers)==0:
        log.info("No cloud workers to check")
    else:
        for cw in cworkers:            
            log.info("Checking cloud worker %s @ %s, computing batch %s on %s" \
                % (cw['cloud_worker_id'],cw['cloud_service_id'],cw['batch_id'],cw['dg_id']))

            stop_cw=False
            if oracle_rpc.is_compute_finished(cw['batch_id']):                
                
                log.info("Cloud worker %s finnished to compute batch %s, stopping it..." % (cw['cloud_worker_id'],cw['batch_id']))
                stop_cw=True

            else:
                log.info("Cloud worker still have to run... Billing ressources spent")
                b=creditsystem_rpc.bill(cw['batch_id'],CREDIT_SPENT_EACH_TURN)
                
                if b==None:
                    log.error("No result from bill(%s), stopping this cloud worker as a precaution" % cw['batch_id'])
                    stop_cw=True               
                elif b==False:
                    log.info("No more credits available for batch %s, stopping cloud worker %d" % (cw['batch_id'],cw['cloud_worker_id']))
                    stop_cw=True
                else:
                    log.info("Done!")

            if stop_cw==True:

                log.info("Stopping CloudWorker %d" % cw['cloud_worker_id'])
                
                r = cloudworker.stop_cloud_worker(cw['cloud_service_id'],cw['cloud_service_conf_file'],cw['dg_id'],cw['dg_conf_file'],cw['batch_id'],cw['batch_data'],cw['cloud_worker_id'],cw['cloud_worker_data'],cw['handler'])
                if r==False:
                    log.error("Error when stopping CloudWorker !! Should be stopped manually !")
                else:

                    log.info("CloudWorker stopped, updating DB status")
                    db.sql_update("CloudWorker",('batch_id','cloud_worker_data'),("NULL","NULL"),"cloud_worker_id=%s" % (cw['cloud_worker_id']))
                    
                    log.info("Checking if there is other cloud workers working on this batch...")
                    other_cworkers = db.sql_select("CloudWorker",('cloud_worker_id',),"batch_id='%s'" % cw['batch_id'])
                    
                    if len(other_cworkers)>0:
                        log.info("Other cloud workers are still working on this batch, doing nothing more")
                    else:
                        log.info("No more cloud workers are working on this batch, asking the credit system to pay the batch order")
                        p=creditsystem_rpc.pay(cw['batch_id'])               
                        if (p==None):
                            log.error("Error when calling pay() function! QoSOrder has to be closed manually")
                        
                        log.info("Removing QoS Order from scheduler database")
                        db.sql_delete("QoSBatch","batch_id='%s'" % cw['batch_id'])
                            
                        #TODO: Test this part 
                        if oracle_rpc.is_compute_finished(cw['batch_id']):                
                            log.info("Unconfigure grid %s for QoS on batch %s" % (cw['dg_id'],cw['batch_id']))
                            dg_rpc.unconfigure_QoS(cw['batch_id'],cw['dg_id'],cw['dg_conf_file'],cw['batch_data'])

            log.info("Cloud worker %s checked" % cw['cloud_worker_id'])


if __name__=="__main__":
    #Credits cost each time a cloud worker is seen running by monitor_cloud_workers() 
    CREDIT_SPENT_EACH_TURN=1

    
    db = db_func.SpeQuloS_DB(os.path.join(sys.path[0],"db.cfg"))        
    monitor_cloud_workers(db)



