#! /usr/bin/python

"""//Scheduler implementation to remotely call Oracle functions//

- File: oracle_rpc.py

"""

import sys
import os
sys.path.append(os.path.join(sys.path[0],".."))
os.chdir(sys.path[0])
from tools import web_func
from tools import conf_func
from tools import log_func
log=log_func.get_spequlos_log(__file__)


#Change this to specify the current time of the system (in Unix ticks) 
CURRENT_TIME=None


def calculate_cloud_benefit(batch_id):
    """//Remote call of Oracle's calculate_cloud_benefit//
    
    - > The function calls and retrieves the result of the calculate_cloud_benefit function from the Oracle web server.
    It needs a file named "oracle.cfg" containing the line "address=<url>", with <url> the url to access to the Oracle web server, to be present in the directory. 
    
    - Arguments:
     - batch_id -- The batch identifier, as a string
    
    - The function returns (r,c), with r the ratio between the expected batch makespan (with tail problem) with batch makespan when using cloud ressources, as a float and c the exepected cost of cloud ressource usage, as an integer
    
    """ 
    if not isinstance(batch_id,str):
        log.error("calculate_cloud_benefit() batch_id argument must be a string")
        return None
             
    params=conf_func.read_config("oracle.cfg")
    
    url="http://"+params['address']+"/calculate_cloud_benefit.py?batch_id="+batch_id\
        +"%s" % "" if CURRENT_TIME is None else "current_time="+str(CURRENT_TIME)

    res=web_func.call(url)
                                                    
    if len(res)!=4:
        log.error("There should be 4 results from url %s" % url)
        return
    try:
        r1 = int(res[1])
        r2 = int(res[2])
        c = int(res[3])
        if r1<0:
            raise ValueError("2nd result is negative")
        if r2<0:
            raise ValueError("3rd result is negative")
        if c<0:
            raise ValueError("4th result is negative")
    except ValueError as e:
        log.error("calculate_cloud_benefit results must be positive integers")            
        log.error(e)            
        return None
    else:
        return float(r1)/r2,c



def get_percent_completed(batch_id):
    """//Remote call of Oracle's get_percent_completed//
    
    - > The function calls and retrieves the result of the get_percent_completed function from the Oracle web server.
    It needs a file named "oracle.cfg" containing the line "address=<url>", with <url> the url to access to the Oracle web server, to be present in the directory. 
    
    - Arguments:
     - batch_id -- The batch identifier, as a string
    
    - The function returns the batch completion ratio, as a float
    
    """ 
    if not isinstance(batch_id,str):
        log.error("get_percent_completed() batch_id argument must be a string")
        raise Exception("get_percent_completed() batch_id argument must be a string")

    #Change this to specify the current time of the system (in Unix ticks) 
    CURRENT_TIME=None             

    params=conf_func.read_config("oracle.cfg")
    
    url="http://"+params['address']+"/get_percent_completed.py?batch_id="+batch_id\
        +"%s" % "" if CURRENT_TIME is None else "current_time="+str(CURRENT_TIME)

    res=web_func.call(url)
                                                    
    if len(res)!=1:
        log.error("There should be 1 results from url %s" % url)
        return
    try:
        if res[0] == "None":
            log.error("Oracle get_percent_completed did not returned result")            
            return None        
        p = float(res[0])
        if p<0:
            raise ValueError("completion is negative")
        if p>1:
            raise ValueError("completion is greater than 1")
    except ValueError as e:
        log.error("get_percent_completed result must be a float between 0 and 1")            
        log.error(e)            
        return None
    else:
        return p


def is_compute_finished(batch_id):
    if get_percent_completed(batch_id)==1:
        return True
    else:
        return False
